################################################################################
# Licensed Materials - Property of IBM Copyright IBM Corporation 2012. All Rights Reserved.
# U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP
# Schedule Contract with IBM Corp.
################################################################################

require "erb" # Ruby templating engine
require "json" # JSON parsing
require 'open-uri'
require 'openssl' 
require 'zip/zip'
require 'zip/zipfilesystem'

################################################################################
# Class Definition                                                             #
################################################################################
module DevopsDeploymentLib
  module Util

    ################################################################################
    # JSON Methods                                                                 #
    ################################################################################
    
    # A method that till take a JSON string or file and convert its contents to 
    # a Hash object.
    #
    # == Params
    #
    # === Required
    # [json (String|URI String)]  A JSON formatted string or URI to a local or remote
    #                             JSON file.
    #
    # == Returns
    # [Hash]  A hashmap object representing the key-value pairs of the original JSON content.
    #
    def create_hash_from_json(json)
      begin
        json_obj = nil
        json_obj = JSON.parse(json)
    
        return json_obj
    
      rescue Exception => bang
    
        json_content = nil
    
        if File.file?(json) or !(json =~ URI::regexp).nil?
          json_content = open(json) { |f| f.read }
        else
          return nil
        end
    
        return JSON.parse(json_content)
      
      end
    end
    
    # Determine if a string is JSON formatted
    #
    # == Params
    #
    # === Required
    # [str (String)]  A string to test for JSON formatting
    #
    # == Returns
    # * True if the provided string is valid JSON
    # * False if the provided string is not valid JSON
    #
    def is_json(str)
      json_content = str
      
      if File.file?(str) or !(str =~ URI::regexp).nil?
        json_content = open(str) { |f| f.read }
      end
      
      begin
        JSON.parse(json_content)
        return true
      rescue 
        return false
      end
      
    end
    
    
        
    # Method used to download a file from a URL
    #
    # == Params
    # FileName: name of file to be created
    # URL: Location of download artifact
    #
    # === Required
    # [str (String)]  A string to test for JSON formatting
    #
    # == Returns file
    #
    def get_file(fileName, url)
      file = nil
      open(fileName, 'wb') do |file|
        file << open(url).read
      end
      unless file.nil?
        return file
      end
      puts "Could not read URL #{url}. The jython command line interface will not be available."
    end
    
    # Method used to download a file from a URL, with Ruby 1.9 SSL extension
    #
    # == Params
    # FileName: name of file to be created
    # URL: Location of download artifact
    # SSL verify mode
    #
    # === Required
    # [str (String)]  A string to test for JSON formatting
    #
    # == Returns file
    #
    def get_file(fileName, url, ssl_verify_mode)
      file = nil
	  #puts "About to write #{fileName} from #{url} mode #{ssl_verify_mode}"
      File.open(fileName, 'wb') do |fo|
  		fo.print open(url, :ssl_verify_mode => ssl_verify_mode).read
  		file = File.new(fileName)
  		#puts "Read #{fileName}, size #{file.size}"
	  end
      
      unless file.nil?
        return file
      end
      puts "Could not read URL #{url}. The jython command line interface will not be available."
    end
    
    # Method used to parse folder name
    #
    # == Params
    #   filePath: name of file
    #   destination: location of the file
    # === Required
    # 
    #
    # == Returns filepath
    
    def split_filename(filePath, startFolder)
      ss = filePath.index(startFolder)
      startFolder = startFolder.gsub("/","")
      ss = ss + startFolder.length + 1
      filePath = filePath[ss, filePath.length] 
      puts(filePath)
     return filePath 
    end
    
        # Method used to zip a file from a given location and file name
    #
    # == Params
    #   file: name of file
    #   destination: location of the file
    # === Required 
    # [str (String)]  
    #
    # == Returns archive
    
    def compress_file (archive_name, entry_name, file_path)
    Zip::ZipFile.open(archive_name, Zip::ZipFile::CREATE) do |zipfile|
    zipfile.get_output_stream(entry_name) do |out_file|
      File.open(file_path) do |in_file|
        while blk = in_file.read(1024**2)
          out_file << blk
        end
      end
    end
    end
    end
  
    # Method used to zip a file from a given location and file name
    #
    # == Params
    #   file: name of file
    #   destination: location of the file
    # === Required 
    # [str (String)]  
    #
    # == Returns archive
    
    def append_compress_file (archive, file, archive_path, file_name)
           Zip::ZipFile.open(archive, 'w') do |zipfile|
         zipfile.add(file_name,file.path)
         return archive
      end
    end

    # Method used to unzip a file from a given location
    #
    # == Params
    #   file: name of file
    #   destination: location of the file
    # === Required
    # [str (String)] 
    #
    # == Returns archive
  
  def unzip_file (file, destination)
    Zip::ZipFile.open(file) { |zip_file|
      zip_file.each { |f|
        f_path=File.join(destination, f.name)
        # puts "unzip_file extracting #{f} into #{f_path}"
        FileUtils.mkdir_p(File.dirname(f_path))
        zip_file.extract(f, f_path) unless File.exist?(f_path)
      }
    }
    return file
  end 
  
    # Method used to zip a file and add to an archive
    #
    # == Params
    #   archive: name of zip file
    #   file: location of the file to be zipped
    #   archive_path: path in the archive 
    # === Required
    # 
    #
    # == Returns archive
    
    
   def append_compress (archive, file, archive_path)
   Zip::ZipFile.open(archive, 'w') do |zipfile|    
        zipfile.add(archive_path + "/" + file,file)
      end
      return archive
   end
   
    # Method used to zip a directory 
    #
    # == Params
    #   path: fully qualified location of directory
    #   start_folder_location: root location folder
    #   archive_name: name of archive
    # remove_file - append to current archive or use a new one
    # === Required
    # 
    #
    # == Returns archive
   
   
   def compress(path, start_folder_location, archive_name, remove_file)
    if (remove_file)
     FileUtils.rm archive_name, :force=>true
    end
    Zip::ZipFile.open(archive_name, 'w') do |zipfile|
       Dir.glob(path + "**/**/") do |subdir|
            Dir.foreach(subdir) do |file|
              puts "Processing .. #{file.inspect}"
              src_file = subdir + file;
              entry = split_filename(subdir, start_folder_location) + file 
              unless(File.directory?(entry)) 
                puts "Compressing #{src_file} at location #{entry}"    
                zipfile.add(entry, src_file)
              end
            end
        end
    end
    return archive_name
    end


    ################################################################################
    # Template Methods                                                             #
    ################################################################################
    
    # Evaluate an ERB template by combining the template string to the values declared
    # in the context object.
    #
    # == Params
    #
    # === Required
    # [template (String)]  An ERB formatted template string.
    # [context (Hash)]  A hashmap that contains the values for the key variables declared 
    #                   in the template string.
    #
    # == Returns
    # [String]  A complete template with its original variables filled in.  
    #
    def evaluate_template(template, environment_obj)
  
      begin
        json_template = ERB.new(template)
        return json_template.result(environment_obj.get_binding_obj)
      rescue Object => e
        puts e.inspect
        puts e.backtrace.join("\n")
        raise StandardError, "The template failed to be evaluated!"
      end
  
    end
  
  end
end
