################################################################################
# Licensed Materials - Property of IBM Copyright IBM Corporation 2012. All Rights Reserved.
# U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP
# Schedule Contract with IBM Corp.
################################################################################

require "devops_library/providers/library_system_provider"
 

module DevopsLibrary
  module Providers

    # Provides a fake (mock) RAMLibraryProvider that acts as the real thing
    # but doesn't actually interact with RAM (Rational Asset Manager)
    class MockRAMLibraryProvider < LibrarySystemProvider
      
      MOCK_URL = "http://0.0.0.0/mockasset"  # A fake url where we predend the asset is saved

      # See RAMLibraryProvider#initialize
      def initialize(args, build=nil, log=STDOUT, log_level=Logger::WARN)
         return nil
      end


      # See RAMLibraryProvider#create_package
      def create_package(package_id, package_version, metadata={}, packages=nil, optimize=true)
        return Struct.new(:id, :version, :metadata, :url).new(package_id, package_version, metadata, MOCK_URL)
      end


      # See RAMLibraryProvider#publish_package
      def publish_package(package, file, filename=nil) 
        return MOCK_URL
      end


      # See RAMLibraryProvider#fetch_package
      def fetch_package(package_id, package_version, folder, metadata={})
        return MOCK_URL
      end 


      # See RAMLibraryProvider#fetch_package_file
      def fetch_package_file(package_id, package_version, filename, folder, metadata={})
        return MOCK_URL
      end 
      
      # See RAMLibraryProvider#fetch_package_files
      def fetch_package_files(package_id, package_version, filename, folder) 
        return MOCK_URL
      end
       

      
      # See RAMLibraryProvider#publish_dependency_file
      def publish_dependency_file(package)
        return MOCK_URL
      end

      
      # See RAMLibraryProvider#get_related
      def get_related(package_id, package_version, related_types=nil, recursive=true)
         return nil
      end

      
      # See RAMLibraryProvider#resolve_package
      def resolve_package(package_id, package_version, metadata)
        return nil
      end

      
      # See RAMLibraryProvider#add_attribute
      def add_attribute(package, attribute_name, attribute_value, attribute_url=nil)
        return MOCK_URL
      end

      
      # See RAMLibraryProvider#get_package
      def get_package(package_id, package_version)
        return nil
      end

       
      # See RAMLibraryProvider#update_package
      def update_package(package_id, package_version, metadata)
        return nil
      end

      
      # See RAMLibraryProvider#get_url
      def get_url(id, version, filename=nil)
        return MOCK_URL
      end

            
      # See RAMLibraryProvider#relate_package   
      def relate_package(package_id, package_version, related_package_id, related_package_version, relationship)
        return nil
      end
      
      
      # See RAMLibraryProvider#categorize_package   
      def categorize_package(package_id, package_version, category, recategorize=true)
        return nil
      end

      
      # See RAMLibraryProvider#query_packages
      def query_packages(query, sort=nil, ascending=true)
        return MOCK_URL
      end
      
      # See RAMLibraryProvider#publish_link
      def publish_link(package, url, name=nil)
        return MOCK_URL
      end
      
      # See RAMLibraryProvider#mark_deployed 
      def mark_deployed(package)
        return nil
      end
      
      # See RAMLibraryProvider#mark_tested 
      def mark_tested(package, test_status)
       return nil
      end
     
    end # end class MockRAMLibraryProvider
  end # end module Providers
end # end module DevopsLibrary 
