module Weaver
  module Core
    
    # A mixin for classes that may contain {Artifact}
    # 
    # Author:: Fabio Oliveira
    module ArtifactContainerMixin
      # Let all classes that include ArtifactContainerMixin have a method to return their policies
      def self.included(baseclazz)
        baseclazz.class_exec {
          def artifacts
            # Since mixins have no initialize() to set up @artifacts, initialize @artifacts lazily
            @artifacts ||= {} unless frozen?
            return @artifacts || {}
          end
        }
        
        baseclazz.register_initialize_copy_handler(:initialize_copy_artifacts)
        baseclazz.register_child_enumerator(:artifacts)           
      end
      
      def initialize_copy_artifacts(other_artifact_container)
        # At this point, self.initialize_copy should have cloned @artifacts but the values are shallow.
        @artifacts = {}
        other_artifact_container.artifacts.each do | id, artifact |
          dup = artifact.clone()
          dup.scope = self
          @artifacts[id] = dup
        end
      end
    end
    
  end
end