require 'weaver/core/base.rb'

module Weaver
  module Core
    
    # A mixin for classes such as Node which may contain LoadDrivers
    # 
    # Author:: Thomas Gschwind <thg@zurich.ibm.com>
    # Copyright:: IBM
    module LoadDriverContainerMixin
      # Let all classes that include LoadDriverContainerMixin have a method to return their load_drivers
      def self.included(baseclazz)
        baseclazz.class_exec {
          def load_drivers
            # Since mixins have no initialize() to set up @load_drivers, initialize @load_drivers lazily
            @load_drivers ||= {} unless frozen?
            return @load_drivers || {}
          end
        }
        
        baseclazz.register_initialize_copy_handler(:initialize_copy_load_drivers)
        #baseclazz.register_validate_handler(:validate_load_drivers)
        baseclazz.register_child_enumerator(:load_drivers)
      end
      
      def initialize_copy_load_drivers(other_load_driver_container)
        # At this point, self.initialize_copy should have cloned @load_drivers but the values are shallow.
        @load_drivers = {}
        other_load_driver_container.load_drivers.each do | id, load_driver |
          dup = load_driver.clone()
          dup.scope = self
          @load_drivers[id] = dup
        end
      end
      
      def validate_load_drivers(out_messages, desired_phases = nil)
        load_drivers.each { |id, load_driver| load_driver.validate_collect(out_messages, desired_phases) }
      end
      
    end

  end
end