require 'weaver/core/base.rb'

module Weaver
  module Core
    # A mixin for classes such as Node which may contain Service
    # 
    # Author:: Ed Snible
    # Copyright:: IBM
    module ServiceContainerMixin
      include Weaver::Core      # Allow us to use @@logger
      
      # Let all classes that include ServiceContainerMixin have a method to return their services
      def self.included(baseclazz)
        baseclazz.class_exec {
          # Hash of id -> Service
          def services
            # Since mixins have no initialize() to set up @services, initialize @services lazily
            @services ||= {} unless frozen?
            return @services || {}
          end
          
          # * +new_services+ Hash of id -> Service
          def services=(new_services)
            @services = new_services
          end
        }
        
        baseclazz.register_initialize_copy_handler(:initialize_copy_services)
        baseclazz.register_validate_handler(:validate_services)
        baseclazz.register_child_enumerator(:services)           
        
      end
      
      # Deeply copy @services
      def initialize_copy_services(other_service_container)
        # At this point, self.initialize_copy should have cloned @services but the values are shallow.
        @services = {}
        other_service_container.services.each do |id, service|
          dup = service.clone()
          dup.scope = self
          @services[id] = dup
        end
      end
      
      # Validate @services
      # * +out_messages+ ProblemStatus will be pushed onto this for any validation message
      # * +desired_phases+ optional Array which may be +[:predefined]:
      def validate_services(out_messages, desired_phases = nil)
        services.each { |id, service| service.validate_collect(out_messages, desired_phases) }
      end
    end
    
  end
end