require 'weaver/core/logging'
require 'weaver/core/topology'

module Weaver
  module Dsl
    # Mix in ImportContainerStatement in statements that may import
    #
    # Author:: Ed Snible
    # Copyright:: IBM
    # @see Weaver::Core::Import
    module ImportContainerStatement
      require 'weaver/dsl/import_statement'
      include Weaver::Core
      
      # Parse +import+ language statement
      def import(filespec_with_namespace)
        if filespec_with_namespace.kind_of?(Hash)
          filespec = filespec_with_namespace.keys.to_a[0]
          namespace = filespec_with_namespace.values.to_a[0]
        else
          filespec = filespec_with_namespace
          namespace = nil
        end
        
        @@logger.debug "import['#{filespec}'] relative to #{dir.inspect}"
        
        import_statement = Weaver::Dsl::ImportStatement.new(namespace.nil? ? filespec : namespace, self)
        begin
          @parser.parse_using_load_path(filespec, dir, import_statement)
        rescue SystemCallError => sce
          @@logger.warn "import failed #{sce.to_s}"
          model.add_validation_failure(sce.message, weaver_stackframe(sce.backtrace))
        end
        
        model.import(namespace, import_statement.model, filespec)
      end
      
    end

  end

end