module Weaver
  module Dsl
    
    # Mix in LoadDriverContainer in statements that may contain {LoadDriverStatement}s
    # 
    # Author:: Thomas Gschwind <thg@zurich.ibm.com>
    # Copyright:: IBM
    module LoadDriverContainer
      include Weaver::Core
      
      # * *Args* :
      #   -+base+ the AbstractStatement being mixed with
      def self.included(base)
        base.register_statement_handler([:load_driver_handler])
      end

      # * *Args* :
      #   -+sym_or_model+ <Symbol representing Weaver type, e.g. <code>:load_driver</code> | Core::LoadDriver
      #   -+id+ Symbol naming entity
      #   -+block+ optional block of additional metadata
      def load_driver_handler(sym_or_model, id, &block)
        return nil unless is_a?(Weaver::Dsl::LoadDriverContainer) 
        return nil unless sym_or_model.kind_of?(LoadDriver) || root_statement.model.weaver_extended_type?(sym_or_model, LoadDriver)
        return nil unless id.kind_of?(Symbol)
         
        prototype = sym_or_model.kind_of?(Symbol) ? root_statement.model.prototype_instance(sym_or_model, id, model, weaver_stackframe) : sym_or_model
        load_driver(id, prototype, &block)
      end
      
      # TODO: check with Ed whether this is fine
      def copy_attributes(target, source)
        target.amd={}
        source.amd.each { |id, attribute|
          dup = attribute.clone()
          dup.scope = target
          target.amd[id] = dup
        }
      end

      # If prototype is nil, we will create a core LoadDriver, otherwise inherit attributes,
      # default attribute values, and validators from the prototype
      def load_driver(id, prototype = nil, &block)
        @@logger.debug "load_driver[(#{prototype.id.to_s+') ' unless prototype.nil?}:#{id}]"

#        # Determine the name for the load driver containing default
#        # settings
#        default_ld_name = prototype.type.to_s
#        if default_ld_name =~ /load_driver$/
#          default_ld_name += "_defaults"
#          default_ld_name = default_ld_name.intern
#        else
#          default_ld_name = nil
#        end
        
        # Go up the model (via the parent) to
        # * find the node statement: necessary for LoadDrivers that need
        #   the node (e.g., its IP address)
        # * find the next upper default load_driver statement: to copy the
        #   default configuration from
        default_ld = nil
        current = self
        node = nil
        while current != nil do
          if current.kind_of? NodeStatement
            node = current
          end
#          unless default_ld_name.nil?
#            p = current.model.child_entities[default_ld_name]
#            if (default_ld.nil?) && (!p.nil?) && (p.kind_of? LoadDriver) && (p.type == prototype.type)
#              default_ld = p
#            end
#          end
          
          if current.respond_to? :parent
            current = current.parent
          else
            break
          end
        end
        
        load_driver = LoadDriverStatement.new(id, self)
        unless prototype.nil?
#          unless default_ld.nil?
#            copy_attributes(prototype, default_ld)
#          end
        
          load_driver.model = prototype
          load_driver.model.id = id
          load_driver.model.scope = @model
        end
        
        @model.load_drivers[id] = load_driver.model
        load_driver.instance_eval &block unless block.nil?

        # TODO: obtain the target_port from the component within which we execute
        load_driver.target_host node.ip_address unless node.nil?
        load_driver.target_port 8080
        
        return load_driver.model
      end
      
    end
    
  end
end