require 'weaver/core/logging'
require 'weaver/coordination/coordination_mixin'

module Weaver
  module Execution
    module Actions
    
      class Action
        include Weaver::Coordination::CoordinationMixin
        
        attr_accessor :plan
        
        def initialize
          # extenders should override
        end
        
        def associate_with_plan (plan, id)
          @plan = plan
          @id = id
        end # def associate_with_plan
        
        def execute_action (properties, execute_parameters = {})
          join_coordination()
          pre_execute_coordination
          execute(properties, execute_parameters)
          post_execute_coordination
          complete_coordinated_action()
          # TODO timeout --> abort
        end # def execute
        
        def interrupt ()
          # TODO abort_coordinated_action()
        end # def interrupt
        
        def pre_execute_coordination
          return if plan.nil? or plan.transaction.nil?
          
          # extenders can override behavior, typically to wait for some other action to complete
        end # def pre_coordination
        
        def execute (properties, execute_parameters = {})
          # extenders can override to implement function
        end # def execute_impl

        def post_execute_coordination
          return if plan.nil? or plan.transaction.nil?
          
          #extenders can override behavior, typically to write a specific completion status
        end # def post_coordination
        
        
      end #class Action
    
    end # module Actions
  end #module Execution
end # module Weaver